;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;                                                                  ;;;
;;; Confidential and proprietary information of ITA Software, Inc.   ;;;
;;;                                                                  ;;;
;;; Copyright (c) 2007-2009 ITA Software, Inc.  All rights reserved. ;;;
;;;                                                                  ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :asdf)

(defparameter *composite-module-names* t
  "Does ASDF know how to decompose module names into pathname components?
The mainline ASDF doesn't, but this patch does it.")

(defun split-path-string (s &optional force-directory)
  (check-type s string)
  (let* ((components (split s nil "/"))
         (last-comp (car (last components))))
    (multiple-value-bind (relative components)
        (if (equal (first components) "")
          (values :absolute (cdr components))
          (values :relative components))
      (cond
        ((equal last-comp "")
         (values relative (butlast components) nil))
        (force-directory
         (values relative components nil))
        (t
         (values relative (butlast components) last-comp))))))

(defmethod component-relative-pathname ((component module))
  (or (slot-value component 'relative-pathname)
      (multiple-value-bind (relative path) (split-path-string (component-name component) t)
        (make-pathname
         :directory `(,relative ,@path)
         :host (pathname-host (component-parent-pathname component))))))

(defmethod component-relative-pathname ((component source-file))
  (multiple-value-bind (relative path name) (split-path-string (component-name component))
    (let ((type (source-file-type component (component-system component)))
          (relative-pathname (slot-value component 'relative-pathname))
          (*default-pathname-defaults* (component-parent-pathname component)))
      (if relative-pathname
	(merge-pathnames
         relative-pathname
         (if type
           (make-pathname :name name :type type)
           name))
        (make-pathname :directory `(,relative ,@path) :name name :type type)))))
